@@ -2,10 +2,12 @@ package V;
 use strict;
 
 use vars qw( $VERSION $NO_EXIT );
-$VERSION  = "0.17";
+$VERSION  = "0.18";
 
 $NO_EXIT ||= 0; # prevent import() from exit()ing and fall of the edge
 
+our $DEBUG ||= $ENV{PERL_V_DEBUG} || 0;
+
 =head1 NAME
 
 V - Print version of the specified module(s).
@@ -24,7 +26,9 @@ modules without loading them:
     require V;
     printf "%s has version '%s'\n", "V", V::get_version( "V" );
 
-Starting with version B<0.17>, V will show all C<package>s or C<class>es in a file.
+Starting with version B<0.17>, V will show all C<package>s or C<class>es in a
+file that have a version. If one wants to see all packages/classes from that
+file, set the environment variable C<PERL_V_SHOW_ALL> to a I<true> value.
 
 If you want all available files/versions from C<@INC>:
 
@@ -90,7 +94,12 @@ The name of the C<package>/C<class>.
 
 =item I<version>
 
-The version for that C<package>/C<class>.
+The version for that C<package>/C<class>. (Can be absent if C<$PERL_V_SHOW_ALL>
+is true.)
+
+=item I<ord>
+
+The ordinal number of occurrence in the file.
 
 =back
 
@@ -247,6 +256,7 @@ sub all_installed {
         }
     }
 
+    do {print {*STDERR} "# $file: @{[scalar $_->version]}\n" for @modules} if $V::DEBUG;
     return @modules;
 }
 
@@ -276,7 +286,7 @@ sub version {
 
         next if $cur_pkg =~ m{^V::Module::Info};
 
-        if (m/^[^=]*([\$*])(([\w\:\']*)\bVERSION)\b.*\=(?!~)/) {
+        if (m/(?:our)?\s*([\$*])(([\w\:\']*)\bVERSION)\s*\=(?![=~])/) {
             { local($1, $2); ($_ = $_) = m/(.*)/; } # untaint
             my ($sigil, $name) = ($1, $2);
             next if m/\$$name\s*=\s*eval.+\$$name/;
@@ -291,7 +301,7 @@ sub version {
             };
         }
         # perl 5.12.0+
-        elsif (m/^\s* (?:package|class) \s+ [^\s]+ \s+ ([^;\{]+) [;\{]/x) {
+        elsif (m/^\s* (?:package|class) \s+ [^\s]+ \s+ (v?[0-9.]+) \s* [;\{]/x) {
             my $ver = $1;
             if ( $] >= 5.012000 ) {
                 $eval{$cur_pkg}{prg} = qq{
@@ -315,9 +325,11 @@ sub version {
     while (my ($pkg, $dat) = each(%eval)) {
         my $result;
         if ($dat->{prg}) {
+            print {*STDERR} "# $pkg: $dat->{prg}\n" if $V::DEBUG;
             local $^W = 0;
             $result = eval($dat->{prg});
-            warn "Could not eval '$dat->{prg}' in $parsefile: $@" if $@;
+            warn("Could not eval '$dat->{prg}' in $parsefile: $@")
+                if $@ && $V::DEBUG;
 
             # use the version modulue to deal with v-strings
             require version;
@@ -332,6 +344,9 @@ sub version {
             }
         );
     }
+    if (! $ENV{PERL_V_SHOW_ALL}) {
+        @results = grep { exists($_->{version}) } @results;
+    }
 
     if (@results > 1) {
         @results = grep {
